window.Gleam.initSearch([{"doc":"qcheck","title":"qcheck","content":"# qcheck\n\nQuickCheck-inspired property-based testing with integrated shrinking for [Gleam](https://gleam.run/).\n\nRather than specifying test cases manually, you describe the invariants that values of a given type must satisfy (&quot;properties&quot;). Then, generators generate lots of values (test cases) on which the properties are checked. Finally, if a value is found for which a given property does not hold, that value is &quot;shrunk&quot; in order to find an nice, informative counter-example that is presented to you.\n\nWhile there are a ton of great articles introducing quickcheck or property-based testing, here are a couple general resources that you may enjoy:\n\n- [An introduction to property based testing](https://fsharpforfunandprofit.com/pbt/)\n- [What is Property Based Testing?](https://hypothesis.works/articles/what-is-property-based-testing/)\n\nYou might also be interested in checking out [this project](https://github.com/mooreryan/gleam_stdlib_testing) that uses qcheck to test Gleam&#39;s stdlib.\n\n## Usage &amp; Examples\n\n- See the API docs for detailed usage,\n- See [qcheck_viewer](https://mooreryan.github.io/gleam_qcheck/) to visualize the distributions of some of the qcheck generators.\n\n### Basic example\n\nHere is a short example to get you started. It assumes you are using [gleeunit](https://github.com/lpil/gleeunit) to run the tests, but any test runner that reasonably handles panics will do.\n\n```gleam\nimport qcheck\n\npub fn int_addition_commutativity__test() {\n  use n &lt;- qcheck.given(qcheck.small_non_negative_int())\n  should.equal(n + 1, 1 + n)\n}\n\npub fn int_addition_commutativity__failures_shrink_to_zero__test() {\n  use n &lt;- qcheck.given(qcheck.small_non_negative_int())\n  should.not_equal(n + 1, 1 + n)\n}\n```\n\nThat second example will fail with an error that may look something like this if you are targeting Erlang.\n\n```\n Failures:\n\n  1) examples/basic_example_test.small_non_negative_int__failures_shrink_to_zero__test\n     Failure: &lt;&lt;&quot;TestError[original_value: 5; shrunk_value: 0; shrink_steps: 1; error: Errored(\n                  atom.create_from_string(\\&quot;assertNotEqual\\&quot;)(\n                    [Module(GleeunitFfi), Line(17), Expression([65, 99, 116, 117, 97, 108]), Value(6)]\n                  )\n                );]&quot;&gt;&gt;\n     stacktrace:\n       qcheck_ffi.fail\n```\n\n- `qcheck.given` sets up the test\n  - If a property holds for all generated values, then `qcheck.given` returns `Nil`.\n  - If a property does not hold for all generated values, then `qcheck.given` will panic.\n- `qcheck.small_non_negative_int()` generates small integers greater than or equal to zero.\n- `should.equal(n + 1, 1 + n)` is the property being tested in the first test.\n  - It should be true for all generated values.\n  - The return value of `qcheck.given` will be `Nil`, because the property does hold for all generated values.\n- `should.not_equal(n + 1, 1 + n)` is the property being tested in the second test.\n  - It should be false for all generated values.\n  - `qcheck.given` will be panic, because the property does not hold for all generated values.\n\n### In-depth example\n\nHere is a more in-depth example. We will create a simple `Point` type, write some serialization functions, and then check that the serializing round-trips.\n\nFirst here is some code to define a `Point`.\n\n```gleam\ntype Point {\n  Point(Int, Int)\n}\n\nfn make_point(x: Int, y: Int) -&gt; Point {\n  Point(x, y)\n}\n\nfn point_equal(p1: Point, p2: Point) -&gt; Bool {\n  let Point(x1, y1) = p1\n  let Point(x2, y2) = p2\n\n  x1 == x2 &amp;&amp; y1 == y2\n}\n\nfn point_to_string(point: Point) -&gt; String {\n  let Point(x, y) = point\n  &quot;(&quot; &lt;&gt; int.to_string(x) &lt;&gt; &quot; &quot; &lt;&gt; int.to_string(y) &lt;&gt; &quot;)&quot;\n}\n```\n\nNext, let&#39;s write a function that parses the string representation into a `Point`. The string representation is pretty simple, `Point(1, 2)` would be represented by the following string: `(1 2)`.\n\nHere is one possible way to parse that string representation into a `Point`. (Note that this implementation is intentionally broken for illustration.)\n\n```gleam\nfn point_of_string(string: String) -&gt; Result(Point, String) {\n  // Create the regex.\n  use re &lt;- result.try(\n    regex.from_string(&quot;\\\\((\\\\d+) (\\\\d+)\\\\)&quot;)\n    |&gt; result.map_error(string.inspect),\n  )\n\n  // Ensure there is a single match.\n  use submatches &lt;- result.try(case regex.scan(re, string) {\n    [Match(_content, submatches)] -&gt; Ok(submatches)\n    _ -&gt; Error(&quot;expected a single match&quot;)\n  })\n\n  // Ensure both submatches are present.\n  use xy &lt;- result.try(case submatches {\n    [Some(x), Some(y)] -&gt; Ok(#(x, y))\n    _ -&gt; Error(&quot;expected two submatches&quot;)\n  })\n\n  // Try to parse both x and y values as integers.\n  use xy &lt;- result.try(case int.parse(xy.0), int.parse(xy.1) {\n    Ok(x), Ok(y) -&gt; Ok(#(x, y))\n    Error(Nil), Ok(_) -&gt; Error(&quot;failed to parse x value&quot;)\n    Ok(_), Error(Nil) -&gt; Error(&quot;failed to parse y value&quot;)\n    Error(Nil), Error(Nil) -&gt; Error(&quot;failed to parse x and y values&quot;)\n  })\n\n  Ok(Point(xy.0, xy.1))\n}\n```\n\nNow we would like to test our implementation. Of course, we could make some examples and test it like so:\n\n```gleam\nimport gleeunit/should\n\npub fn roundtrip_test() {\n  let point = Point(1, 2)\n  let parsed_point = point |&gt; point_to_string |&gt; point_of_string\n\n  point_equal(point, parsed_point) |&gt; should.be_true\n}\n```\n\nThat&#39;s fine, and you can imagine taking some corner cases like putting in `0` or `-1` or the max and min values for integers on your selected target. Rather, let&#39;s think of a property to test.\n\nI mention round-tripping, but how can you write a property to test it. Something like, &quot;given a valid point, when serializing it to a string, and then deserializing that string into another point, both points should always be equal&quot;.\n\nOkay, first we need to write a generator of valid points. In this case, it isn&#39;t too interesting as any integer can be used for both `x` and `y` values of the point. So we can use `generator.map2` like so:\n\n```gleam\nfn point_generator() {\n  qcheck.map2(qcheck.int_uniform(), qcheck.int_uniform(), make_point)\n}\n```\n\nAlternatively, if you prefer the `use` syntax, you could write:\n\n```gleam\nfn point_generator() {\n  use x, y &lt;- qcheck.map2(qcheck.int_uniform(), qcheck.int_uniform())\n\n  make_point(x, y)\n}\n```\n\nNow that we have the point generator, we can write a property test. (It uses the `gleeunit/should.be_true` function again.)\n\n```gleam\npub fn point_serialization_roundtripping__test() {\n  use generated_point &lt;- qcheck.given(point_generator())\n\n  let assert Ok(parsed_point) =\n    generated_point\n    |&gt; point_to_string\n    |&gt; point_of_string\n\n  should.be_true(point_equal(generated_point, parsed_point))\n}\n```\n\nLet&#39;s try and run the test. (Note that your output won&#39;t look exactly like this.)\n\n```\n$ gleam test\n\n  1) examples/parsing_example_test.point_serialization_roundtripping__test: module &#39;examples@parsing_example_test&#39;\n     Failure: &lt;&lt;&quot;TestError[original_value: Point(-875333649, -1929681101); shrunk_value: Point(0, -1); shrink_steps: 31; error: Errored(dict.from_list([#(Function, \\&quot;point_serialization_roundtripping__test\\&quot;), #(Line, 74), #(Message, \\&quot;Assertion pattern match failed\\&quot;), #(Module, \\&quot;examples/parsing_example_test\\&quot;), #(Value, Error(\\&quot;expected a single match\\&quot;)), #(GleamError, LetAssert)]));]&quot;&gt;&gt;\n     stacktrace:\n       qcheck_ffi.fail\n     output:\n```\n\nThere is a failure. Now, currently, this output is pretty noisy. Here are the important parts to highlight.\n\n- `original_value: Point(-875333649, -1929681101)`\n  - This is the original counter-example that causes the test to fail.\n- `shrunk_value: Point(0, -1)`\n  - Because `qcheck` generators have integrated shrinking, that counter-example &quot;shrinks&quot; to this simpler example.\n  - The &quot;shrunk&quot; examples can help you better identify what the problem may be.\n- `Error(\\&quot;expected a single match\\&quot;))`\n  - Here is the error message that actually caused the failure.\n\nSo we see a failure with `Point(0, -1)`, which means it probably has something to do with the negative number. Also, we see that the `Error(&quot;expected a single match&quot;)` is what triggered the failure. That error comes about when `regex.scan` fails in the `point_of_string` function.\n\nGiven those two pieces of information, we can infer that the issue is in our regular expression definition: `regex.from_string(&quot;\\\\((\\\\d+) (\\\\d+)\\\\)&quot;)`. And now we may notice that we are not allowing for negative numbers in the regular expression. To fix it, change that line to the following:\n\n```gleam\n    regex.from_string(&quot;\\\\((-?\\\\d+) (-?\\\\d+)\\\\)&quot;)\n```\n\nThat is allowing an optional `-` sign in front of the integers. Now when you rerun the `gleam test`, everything passes.\n\nYou could imagine combining a property test like the one above, with a few well chosen examples to anchor everything, into a nice little test suite that exercises the serialization of points in a small amount of test code.\n\n(The full code for this example can be found in `test/examples/parsing_example_test.gleam`.)\n\n### Applicative style\n\nThe applicative style provides a nice interface for creating generators for custom types.\n\n```gleam\nimport qcheck\n\n/// A simple Box type with position (x, y) and dimensions (width, height).\ntype Box {\n  Box(x: Int, y: Int, w: Int, h: Int)\n}\n\nfn box_generator() {\n  // Lift the Box creating function into the Generator structure.\n  qcheck.return({\n    use x &lt;- qcheck.parameter\n    use y &lt;- qcheck.parameter\n    use w &lt;- qcheck.parameter\n    use h &lt;- qcheck.parameter\n    Box(x:, y:, w:, h:)\n  })\n  // Set the `x` generator.\n  |&gt; qcheck.apply(qcheck.int_uniform_inclusive(-100, 100))\n  // Set the `y` generator.\n  |&gt; qcheck.apply(qcheck.int_uniform_inclusive(-100, 100))\n  // Set the `width` generator.\n  |&gt; qcheck.apply(qcheck.int_uniform_inclusive(1, 100))\n  // Set the `height` generator.\n  |&gt; qcheck.apply(qcheck.int_uniform_inclusive(1, 100))\n}\n```\n\n### Integrating with testing frameworks\n\nYou don&#39;t have to do anything special to integrate `qcheck` with a testing framework like [gleeunit](https://github.com/lpil/gleeunit). The only thing required is that your testing framework of choice be able to handle panics/exceptions.\n\n_Note: [startest](https://github.com/maxdeviant/startest) should be fine._\n\nYou may also be interested in [qcheck_gleeunit_utils](https://github.com/mooreryan/qcheck_gleeunit_utils) for running your tests in parallel and controlling test timeouts when using gleeunit and targeting Erlang.\n\n## Acknowledgements\n\nVery heavily inspired by the [qcheck](https://github.com/c-cube/qcheck) and [base_quickcheck](https://github.com/janestreet/base_quickcheck) OCaml packages.\n\n## Contributing\n\nThank you for your interest in the project!\n\n- Bug reports, feature requests, suggestions and ideas are welcomed. Please open an [issue](https://github.com/mooreryan/gleam_qcheck/issues/new/choose) to start a discussion.\n- External contributions will generally not be accepted without prior discussion.\n  - If you have an idea for a new feature, please open an issue for discussion prior to working on a pull request.\n  - Small pull requests for bug fixes, typos, or other changes with limited scope may be accepted. If in doubt, please open an issue for discussion first.\n\n## License\n\n[![license MIT or Apache\n2.0](https://img.shields.io/badge/license-MIT%20or%20Apache%202.0-blue)](https://github.com/mooreryan/gleam_qcheck)\n\nCopyright (c) 2024 - 2025 Ryan M. Moore\n\nLicensed under the Apache License, Version 2.0 or the MIT license, at your option. This program may not be copied, modified, or distributed except according to those terms.\n","url":"index.html"},{"doc":"qcheck/random","title":"Generator","content":"pub opaque type Generator(a)\n\n\nSynonyms:\nqcheck/random.Generator\nqcheck/random Generator","url":"qcheck/random.html#Generator"},{"doc":"qcheck/random","title":"Seed","content":"pub opaque type Seed\n An opaque type representing a seed value used to initialize random generators.\n\n\nSynonyms:\nqcheck/random.Seed\nqcheck/random Seed","url":"qcheck/random.html#Seed"},{"doc":"qcheck/random","title":"max_int","content":"pub const max_int: Int\n Attempting to generate values below this limit will not lead to good random results.\n\nSynonyms:\nqcheck/random.max_int\nqcheck/random max_int","url":"qcheck/random.html#max_int"},{"doc":"qcheck/random","title":"min_int","content":"pub const min_int: Int\n Attempting to generate values below this limit will not lead to good random results.\n\nSynonyms:\nqcheck/random.min_int\nqcheck/random min_int","url":"qcheck/random.html#min_int"},{"doc":"qcheck/random","title":"bind","content":"pub fn bind(\n  generator: Generator(a),\n  f: fn(a) -&gt; Generator(b),\n) -&gt; Generator(b)\n\nSynonyms:\nqcheck/random.bind\nqcheck/random bind","url":"qcheck/random.html#bind"},{"doc":"qcheck/random","title":"choose","content":"pub fn choose(one: a, other: a) -&gt; Generator(a)\n\nSynonyms:\nqcheck/random.choose\nqcheck/random choose","url":"qcheck/random.html#choose"},{"doc":"qcheck/random","title":"constant","content":"pub fn constant(value: a) -&gt; Generator(a)\n\nSynonyms:\nqcheck/random.constant\nqcheck/random constant","url":"qcheck/random.html#constant"},{"doc":"qcheck/random","title":"float","content":"pub fn float(from from: Float, to to: Float) -&gt; Generator(Float)\n\nSynonyms:\nqcheck/random.float\nqcheck/random float","url":"qcheck/random.html#float"},{"doc":"qcheck/random","title":"float_weighted","content":"pub fn float_weighted(\n  first: #(Float, a),\n  others: List(#(Float, a)),\n) -&gt; Generator(a)\n Like `weighted` but uses `Floats` to specify the weights.\n\n Generally you should prefer `weighted` as it is faster.\n\nSynonyms:\nqcheck/random.float_weighted\nqcheck/random float_weighted","url":"qcheck/random.html#float_weighted"},{"doc":"qcheck/random","title":"int","content":"pub fn int(from from: Int, to to: Int) -&gt; Generator(Int)\n\nSynonyms:\nqcheck/random.int\nqcheck/random int","url":"qcheck/random.html#int"},{"doc":"qcheck/random","title":"map","content":"pub fn map(\n  generator: Generator(a),\n  fun: fn(a) -&gt; b,\n) -&gt; Generator(b)\n\nSynonyms:\nqcheck/random.map\nqcheck/random map","url":"qcheck/random.html#map"},{"doc":"qcheck/random","title":"random_sample","content":"pub fn random_sample(generator: Generator(a)) -&gt; a\n\nSynonyms:\nqcheck/random.random_sample\nqcheck/random random_sample","url":"qcheck/random.html#random_sample"},{"doc":"qcheck/random","title":"random_seed","content":"pub fn random_seed() -&gt; Seed\n `random_seed()` creates a new randomly-generated seed.  You can use it when\n you don&#39;t care about having specifically reproducible results.\n\n ### Example\n\n Use a random seed for the `Config`.\n\n \n let config =\n   qcheck.default_config()\n   |&gt; qcheck.with_seed(qcheck.random_seed())\n \n\nSynonyms:\nqcheck/random.random_seed\nqcheck/random random_seed","url":"qcheck/random.html#random_seed"},{"doc":"qcheck/random","title":"sample","content":"pub fn sample(generator: Generator(a), seed: Seed) -&gt; a\n\nSynonyms:\nqcheck/random.sample\nqcheck/random sample","url":"qcheck/random.html#sample"},{"doc":"qcheck/random","title":"seed","content":"pub fn seed(n: Int) -&gt; Seed\n `seed(n) creates a new seed from the given integer, `n`.\n\n ### Example\n\n Use a specific seed for the `Config`.\n\n \n let config =\n   qcheck.default_config()\n   |&gt; qcheck.with_seed(qcheck.seed(124))\n \n\nSynonyms:\nqcheck/random.seed\nqcheck/random seed","url":"qcheck/random.html#seed"},{"doc":"qcheck/random","title":"step","content":"pub fn step(generator: Generator(a), seed: Seed) -&gt; #(a, Seed)\n\nSynonyms:\nqcheck/random.step\nqcheck/random step","url":"qcheck/random.html#step"},{"doc":"qcheck/random","title":"then","content":"pub fn then(\n  generator: Generator(a),\n  f: fn(a) -&gt; Generator(b),\n) -&gt; Generator(b)\n `then` is an alias for `bind`.\n\nSynonyms:\nqcheck/random.then\nqcheck/random then","url":"qcheck/random.html#then"},{"doc":"qcheck/random","title":"to_random_yielder","content":"pub fn to_random_yielder(generator: Generator(a)) -&gt; Yielder(a)\n\nSynonyms:\nqcheck/random.to_random_yielder\nqcheck/random to_random_yielder","url":"qcheck/random.html#to_random_yielder"},{"doc":"qcheck/random","title":"to_yielder","content":"pub fn to_yielder(\n  generator: Generator(a),\n  seed: Seed,\n) -&gt; Yielder(a)\n\nSynonyms:\nqcheck/random.to_yielder\nqcheck/random to_yielder","url":"qcheck/random.html#to_yielder"},{"doc":"qcheck/random","title":"uniform","content":"pub fn uniform(first: a, others: List(a)) -&gt; Generator(a)\n\nSynonyms:\nqcheck/random.uniform\nqcheck/random uniform","url":"qcheck/random.html#uniform"},{"doc":"qcheck/random","title":"weighted","content":"pub fn weighted(\n  first: #(Int, a),\n  others: List(#(Int, a)),\n) -&gt; Generator(a)\n\nSynonyms:\nqcheck/random.weighted\nqcheck/random weighted","url":"qcheck/random.html#weighted"},{"doc":"qcheck/random","title":"qcheck/random","content":" Random\n\n The random module provides basic random value generators that can be used\n to define Generators.\n\n They are mostly inteded for internal use or &quot;advanced&quot; manual construction\n of generators.  In typical usage, you will probably not need to interact\n with these functions much, if at all.  As such, they are currently mostly\n undocumented.\n","url":"qcheck/random.html"},{"doc":"qcheck/shrink","title":"atomic","content":"pub fn atomic() -&gt; fn(a) -&gt; Yielder(a)\n The `atomic` shrinker treats types as atomic, and never attempts to produce\n smaller values.\nSynonyms:\nqcheck/shrink.atomic\nqcheck/shrink atomic","url":"qcheck/shrink.html#atomic"},{"doc":"qcheck/shrink","title":"float_towards","content":"pub fn float_towards(\n  destination: Float,\n) -&gt; fn(Float) -&gt; Yielder(Float)\n\nSynonyms:\nqcheck/shrink.float_towards\nqcheck/shrink float_towards","url":"qcheck/shrink.html#float_towards"},{"doc":"qcheck/shrink","title":"int_towards","content":"pub fn int_towards(destination: Int) -&gt; fn(Int) -&gt; Yielder(Int)\n\nSynonyms:\nqcheck/shrink.int_towards\nqcheck/shrink int_towards","url":"qcheck/shrink.html#int_towards"},{"doc":"qcheck/shrink","title":"qcheck/shrink","content":" Shrinking helper functions\n\n This module contains helper functions that can be used to build custom generators (not by composing other generators).\n\n They are mostly intended for internal use or &quot;advanced&quot; manual construction\n of generators.  In typical usage, you will probably not need to interact\n with these functions much, if at all.  As such, they are currently mostly\n undocumented.\n\n In fact, if you are using these functions a lot, file a issue on GitHub\n and let me know if there are any generator combinators that you&#39;re missing.\n","url":"qcheck/shrink.html"},{"doc":"qcheck/tree","title":"Tree","content":"pub type Tree(a) {\n  Tree(a, Yielder(Tree(a)))\n}\n\nTree(a, Yielder(Tree(a)))\n\n\nSynonyms:\nqcheck/tree.Tree\nqcheck/tree Tree","url":"qcheck/tree.html#Tree"},{"doc":"qcheck/tree","title":"apply","content":"pub fn apply(f: Tree(fn(a) -&gt; b), x: Tree(a)) -&gt; Tree(b)\n\nSynonyms:\nqcheck/tree.apply\nqcheck/tree apply","url":"qcheck/tree.html#apply"},{"doc":"qcheck/tree","title":"bind","content":"pub fn bind(tree: Tree(a), f: fn(a) -&gt; Tree(b)) -&gt; Tree(b)\n\nSynonyms:\nqcheck/tree.bind\nqcheck/tree bind","url":"qcheck/tree.html#bind"},{"doc":"qcheck/tree","title":"collect","content":"pub fn collect(tree: Tree(a), f: fn(a) -&gt; b) -&gt; List(b)\n Collect values of the tree into a list, while processing them with the\n mapping given function `f`.\n\nSynonyms:\nqcheck/tree.collect\nqcheck/tree collect","url":"qcheck/tree.html#collect"},{"doc":"qcheck/tree","title":"map","content":"pub fn map(tree: Tree(a), f: fn(a) -&gt; b) -&gt; Tree(b)\n\nSynonyms:\nqcheck/tree.map\nqcheck/tree map","url":"qcheck/tree.html#map"},{"doc":"qcheck/tree","title":"map2","content":"pub fn map2(a: Tree(a), b: Tree(b), f: fn(a, b) -&gt; c) -&gt; Tree(c)\n\nSynonyms:\nqcheck/tree.map2\nqcheck/tree map2","url":"qcheck/tree.html#map2"},{"doc":"qcheck/tree","title":"new","content":"pub fn new(x: a, shrink: fn(a) -&gt; Yielder(a)) -&gt; Tree(a)\n\nSynonyms:\nqcheck/tree.new\nqcheck/tree new","url":"qcheck/tree.html#new"},{"doc":"qcheck/tree","title":"option","content":"pub fn option(tree: Tree(a)) -&gt; Tree(Option(a))\n\nSynonyms:\nqcheck/tree.option\nqcheck/tree option","url":"qcheck/tree.html#option"},{"doc":"qcheck/tree","title":"return","content":"pub fn return(x: a) -&gt; Tree(a)\n\nSynonyms:\nqcheck/tree.return\nqcheck/tree return","url":"qcheck/tree.html#return"},{"doc":"qcheck/tree","title":"sequence_trees","content":"pub fn sequence_trees(l: List(Tree(a))) -&gt; Tree(List(a))\n `sequence_trees(list_of_trees)` sequences a list of trees into a tree of lists.\n\nSynonyms:\nqcheck/tree.sequence_trees\nqcheck/tree sequence_trees","url":"qcheck/tree.html#sequence_trees"},{"doc":"qcheck/tree","title":"to_string","content":"pub fn to_string(\n  tree: Tree(a),\n  a_to_string: fn(a) -&gt; String,\n) -&gt; String\n `to_string(tree, element_to_string)` converts a tree into an unspecified string representation.\n\n - `element_to_string`: a function that converts individual elements of the tree to strings.\n\nSynonyms:\nqcheck/tree.to_string\nqcheck/tree to_string","url":"qcheck/tree.html#to_string"},{"doc":"qcheck/tree","title":"to_string_max_depth","content":"pub fn to_string_max_depth(\n  tree: Tree(a),\n  a_to_string: fn(a) -&gt; String,\n  max_depth: Int,\n) -&gt; String\n Like `to_string` but with a configurable `max_depth`.\n\nSynonyms:\nqcheck/tree.to_string_max_depth\nqcheck/tree to_string_max_depth","url":"qcheck/tree.html#to_string_max_depth"},{"doc":"qcheck/tree","title":"qcheck/tree","content":" Trees\n\n This module contains functions for creating and manipulating shrink trees.\n\n They are mostly inteded for internal use or &quot;advanced&quot; manual construction\n of generators.  In typical usage, you will probably not need to interact\n with these functions much, if at all.  As such, they are currently mostly\n undocumented.\n\n In fact, if you are using these functions a lot, file a issue on GitHub\n and let me know if there are any generator combinators that you&#39;re missing.\n\n There are functions for dealing with the [Tree](#Tree) type directly, but\n they are low-level and you should not need to use them much.\n\n - The [Tree](#Tree) type\n - [new](#new)\n - [return](#return)\n - [map](#map)\n - [map2](#map2)\n - [bind](#bind)\n - [apply](#apply)\n - [collect](#collect)\n - [sequence_trees](#sequence_trees)\n - [option](#option)\n - [to_string](#to_string)\n - [to_string_with_max_depth](#to_string_with_max_depth)\n","url":"qcheck/tree.html"},{"doc":"qcheck","title":"Config","content":"pub opaque type Config\n Configuration for the property-based testing.\n\n\nSynonyms:\nqcheck.Config\nqcheck Config","url":"qcheck.html#Config"},{"doc":"qcheck","title":"Generator","content":"pub type Generator(a) {\n  Generator(fn(Seed) -&gt; #(Tree(a), Seed))\n}\n A generator for producing random values of type `a`.\n\n While the type is not opaque, it&#39;s recommended to use the provided\n generators, and the provided combinators like `map` and `bind`.  Direct\n generator construction should be reserved for special use-cases.\n\nGenerator(fn(Seed) -&gt; #(Tree(a), Seed))\n\n\nSynonyms:\nqcheck.Generator\nqcheck Generator","url":"qcheck.html#Generator"},{"doc":"qcheck","title":"Seed","content":"pub type Seed =\n  random.Seed\n A type representing a seed value used to initialize random generators.\n\n\nSynonyms:\nqcheck.Seed\nqcheck Seed","url":"qcheck.html#Seed"},{"doc":"qcheck","title":"alphabetic_ascii_codepoint","content":"pub fn alphabetic_ascii_codepoint() -&gt; Generator(UtfCodepoint)\n Generate alphabetic ASCII characters.\n\n ### Returns\n\n A generator that produces alphabetic ASCII characters as codepoints\n\n ### Example\n\n \n string_from(alphabetic_ascii_codepoint())\n \n\nSynonyms:\nqcheck.alphabetic_ascii_codepoint\nqcheck alphabetic_ascii_codepoint","url":"qcheck.html#alphabetic_ascii_codepoint"},{"doc":"qcheck","title":"alphanumeric_ascii_codepoint","content":"pub fn alphanumeric_ascii_codepoint() -&gt; Generator(UtfCodepoint)\n Generate alphanumeric ASCII characters.\n\n ### Returns\n\n A generator that produces alphanumeric ASCII characters as codepoints\n\n ### Example\n\n \n string_from(alphanumeric_ascii_codepoint())\n \n\nSynonyms:\nqcheck.alphanumeric_ascii_codepoint\nqcheck alphanumeric_ascii_codepoint","url":"qcheck.html#alphanumeric_ascii_codepoint"},{"doc":"qcheck","title":"apply","content":"pub fn apply(\n  f: Generator(fn(a) -&gt; b),\n  x: Generator(a),\n) -&gt; Generator(b)\n Support for constructing generators in an applicative style.\n\n ### Example\n\n \n type Box {\n   Box(x: Int, y: Int, w: Int, h: Int)\n }\n\n fn box_generator() {\n   return({\n     use x &lt;- parameter\n     use y &lt;- parameter\n     use w &lt;- parameter\n     use h &lt;- parameter\n     Box(x:, y:, w:, h:)\n   })\n   |&gt; apply(bounded_int(-100, 100))\n   |&gt; apply(bounded_int(-100, 100))\n   |&gt; apply(bounded_int(1, 100))\n   |&gt; apply(bounded_int(1, 100))\n }\n \n\nSynonyms:\nqcheck.apply\nqcheck apply","url":"qcheck.html#apply"},{"doc":"qcheck","title":"ascii_digit_codepoint","content":"pub fn ascii_digit_codepoint() -&gt; Generator(UtfCodepoint)\n Generate ASCII digits as codepoints.\n\n ### Returns\n\n A generator that produces ASCII digits from `0` to `9` as codepoints\n\n ### Example\n\n \n string_from(ascii_digit_codepoint())\n \n\nSynonyms:\nqcheck.ascii_digit_codepoint\nqcheck ascii_digit_codepoint","url":"qcheck.html#ascii_digit_codepoint"},{"doc":"qcheck","title":"ascii_whitespace_codepoint","content":"pub fn ascii_whitespace_codepoint() -&gt; Generator(UtfCodepoint)\n Generate ASCII whitespace as codepoints.\n\n ### Returns\n\n A generator that produces ASCII whitespace as codepoints\n\n ### Example\n\n \n let whitespace_generator = string_from(ascii_whitespace_codepoint())\n \n\nSynonyms:\nqcheck.ascii_whitespace_codepoint\nqcheck ascii_whitespace_codepoint","url":"qcheck.html#ascii_whitespace_codepoint"},{"doc":"qcheck","title":"bind","content":"pub fn bind(\n  generator: Generator(a),\n  f: fn(a) -&gt; Generator(b),\n) -&gt; Generator(b)\n Transform a generator by applying a function that returns another\n generator to each generated value.\n\n Unlike `map`, this allows for a dependency on the resulting generator and\n the original generated values.\n\n ### Arguments\n\n - `generator`: A generator that creates a value of type `a`\n - `f`: A function that takes a value of type `a` and returns a generator\n     of type `b`\n\n ### Returns\n\n A generator that first generates a value of type `a`, then uses that value\n to generate a value of type `b`\n\n ### Examples\n\n Say you wanted to generate a valid date in string form, like `&quot;2025-01-30&quot;`.\n In order to generate a valid day, you need both the month (some months have\n 31 days, other have fewer) and also the year (since the year affects the max\n days in February). So, before you can generate a valid day, you must first\n generate a year and a month. You could imagine a set of functions like this:\n\n \n fn date_generator() -&gt; Generator(String) {\n   use #(year, month) &lt;- bind(tuple2(year_generator(), month_generator()))\n   use day &lt;- map(day_generator(year:, month:))\n\n   int.to_string(year)\n   &lt;&gt; &quot;-&quot;\n   &lt;&gt; int.to_string(month)\n   &lt;&gt; &quot;-&quot;\n   &lt;&gt; int.to_string(day)\n }\n\n // Note how the day generator depends on the value of `year` and `month`.\n fn day_generator(year year, month month) -&gt; Generator(Int) {\n   todo\n }\n\n fn year_generator() -&gt; Generator(Int) {\n   todo\n }\n\n fn month_generator() -&gt; Generator(Int) {\n   todo\n }\n \n\n Another situation in which you would need `bind` is if you needed to\n generate departure and arrival times. We will say a pair of departure and\n arrival times is valid if the departure time is before the arrival time.\n That means we cannot generate an arrival time without first generating a\n departure time. Here is how that might look:\n\n \n fn departure_and_arrival_generator() {\n   use departure_time &lt;- bind(departure_time_generator())\n   use arrival_time &lt;- map(arrival_time_generator(departure_time))\n   #(departure_time, arrival_time)\n }\n\n fn departure_time_generator() {\n   todo\n }\n\n fn arrival_time_generator(departure_time) {\n   todo\n }\n \n\nSynonyms:\nqcheck.bind\nqcheck bind","url":"qcheck.html#bind"},{"doc":"qcheck","title":"bit_array","content":"pub fn bit_array() -&gt; Generator(BitArray)\n Generate bit arrays.\n\n ### Warning\n\n This function will generate bit arrays that cause runtime crashes when\n targeting JavaScript.\n\nSynonyms:\nqcheck.bit_array\nqcheck bit_array","url":"qcheck.html#bit_array"},{"doc":"qcheck","title":"bool","content":"pub fn bool() -&gt; Generator(Bool)\n Generate boolean values.\n\n ### Returns\n\n A generator that generates boolean values and shrinks towards `False`\n\nSynonyms:\nqcheck.bool\nqcheck bool","url":"qcheck.html#bool"},{"doc":"qcheck","title":"bounded_codepoint","content":"pub fn bounded_codepoint(\n  from low: Int,\n  to high: Int,\n) -&gt; Generator(UtfCodepoint)\n Generate Unicode codepoints uniformly distributed within a specified\n range.\n\n ### Arguments\n\n - `from`: Minimum codepoint value (inclusive)\n - `to`: Maximum codepoint value (inclusive)\n\n ### Returns\n\n A generator that produces Unicode codepoints within the specified range.\n\n ### Notes\n\n - If the range is invalid, it will be automatically adjusted to a valid\n   range\n - Shrinks towards an origin codepoint (typically lowercase &#39;a&#39;)\n - Mainly used for string generation\n\n ### Example\n\n \n let cyrillic_character = bounded_codepoint(from: 0x0400, to: 0x04FF)\n \n\nSynonyms:\nqcheck.bounded_codepoint\nqcheck bounded_codepoint","url":"qcheck.html#bounded_codepoint"},{"doc":"qcheck","title":"bounded_float","content":"pub fn bounded_float(\n  from low: Float,\n  to high: Float,\n) -&gt; Generator(Float)\n Generate floats uniformly distributed between `from` and `to`, inclusive.\n\n ### Arguments\n\n - `from`: Lower bound of the range (inclusive)\n - `to`: Upper bound of the range (inclusive)\n\n ### Returns\n\n A generator producing floats within the specified range.\n\n ### Behavior\n\n - Shrinks towards `0`, but won&#39;t shrink outside of the range `[from, to]`\n - Automatically orders parameters if `from` &gt; `to`\n\n ### Example\n\n Generate floats between -10 and 10.\n\n \n bounded_float(-10, 10)\n \nSynonyms:\nqcheck.bounded_float\nqcheck bounded_float","url":"qcheck.html#bounded_float"},{"doc":"qcheck","title":"bounded_int","content":"pub fn bounded_int(from low: Int, to high: Int) -&gt; Generator(Int)\n Generate integers uniformly distributed between `from` and `to`, inclusive.\n\n ### Arguments\n\n - `from`: Lower bound of the range (inclusive)\n - `to`: Upper bound of the range (inclusive)\n\n ### Returns\n\n A generator producing integers within the specified range.\n\n ### Behavior\n\n - Shrinks towards `0`, but won&#39;t shrink outside of the range `[from, to]`\n - Automatically orders parameters if `from` &gt; `to`\n\n ### Example\n\n Generate integers between -10 and 10.\n\n \n bounded_int(-10, 10)\n \n\nSynonyms:\nqcheck.bounded_int\nqcheck bounded_int","url":"qcheck.html#bounded_int"},{"doc":"qcheck","title":"byte_aligned_bit_array","content":"pub fn byte_aligned_bit_array() -&gt; Generator(BitArray)\n Generate byte-aligned bit arrays.\n\nSynonyms:\nqcheck.byte_aligned_bit_array\nqcheck byte_aligned_bit_array","url":"qcheck.html#byte_aligned_bit_array"},{"doc":"qcheck","title":"codepoint","content":"pub fn codepoint() -&gt; Generator(UtfCodepoint)\n Generate Unicode codepoints with a decent distribution that is good for\n generating genreal strings.\n\n ### Returns\n\n A generator that produces Unicode codepoints\n\n ### Example\n\n The decent default string generator could be writen something like this:\n\n \n generic_string(codepoint(), small_non_negative_int())\n \n\nSynonyms:\nqcheck.codepoint\nqcheck codepoint","url":"qcheck.html#codepoint"},{"doc":"qcheck","title":"codepoint_from_ints","content":"pub fn codepoint_from_ints(\n  first: Int,\n  rest: List(Int),\n) -&gt; Generator(UtfCodepoint)\n Generate a codepoint from a list of codepoints represented as integers.\n\n Splitting up the arguments in this way ensures some value is always\n generated by preventing you from passing in an empty list.\n\n ### Arguments\n\n - `first`: First codepoint to choose from\n - `rest`: Additional codepoints to choose from\n\n ### Returns\n\n A `Generator` that produces codepoints from the provided values\n\n ### Example\n\n \n let ascii_whitespace_generator = codepoint_from_ints(\n   // Horizontal tab\n   9,\n   [\n     // Line feed\n     10,\n     // Vertical tab\n     11,\n     // Form feed\n     12,\n     // Carriage return\n     13,\n     // Space\n     32,\n   ],\n )\n \n\nSynonyms:\nqcheck.codepoint_from_ints\nqcheck codepoint_from_ints","url":"qcheck.html#codepoint_from_ints"},{"doc":"qcheck","title":"codepoint_from_strings","content":"pub fn codepoint_from_strings(\n  first: String,\n  rest: List(String),\n) -&gt; Generator(UtfCodepoint)\n Generate a codepoint from a list of strings.\n\n ### Arguments\n\n - `first`: First character to choose from\n - `rest`: Additional characters to choose from\n\n ### Returns\n\n A `Generator` that produces codepoints from the provided values\n\n ### Notes\n\n - Splitting up the arguments in this way ensures some value is always\n   generated by preventing you from passing in an empty list.\n - Only the first codepoint is taken from each of the provided strings\n\n ### Example\n\n \n let quadrant_generator = codepoint_from_strings(&quot;▙&quot;, [&quot;▛&quot;, &quot;▜&quot;, &quot;▟&quot;])\n \n\nSynonyms:\nqcheck.codepoint_from_strings\nqcheck codepoint_from_strings","url":"qcheck.html#codepoint_from_strings"},{"doc":"qcheck","title":"config","content":"pub fn config(\n  test_count test_count: Int,\n  max_retries max_retries: Int,\n  seed seed: Seed,\n) -&gt; Config\n Create a new `Config` with specified test count, max retries, and seed.\n\n ### Arguments\n\n - `test_count`: Number of test cases to generate\n - `max_retries`: Maximum retries to test a shrunk input candidate.\n      Values &gt; 1 can be useful for testing non-deterministic code.\n - `seed`: Random seed for deterministic test generation\n\n ### Returns\n\n A `Config` with the provided settings, using defaults for any invalid arguments\n\n ### Example\n\n \n let config = config(test_count: 10_000, max_retries: 1, seed: seed(47))\n \n\nSynonyms:\nqcheck.config\nqcheck config","url":"qcheck.html#config"},{"doc":"qcheck","title":"constant","content":"pub fn constant(a: a) -&gt; Generator(a)\n Create a generator that always returns the same value and does not shrink.\n\n ### Arguments\n\n - `a`: The value to be consistently generated\n\n ### Returns\n\n A `Generator` that produces the same input for all test cases\n\n ### Example\n\n \n use string &lt;- given(constant(&quot;Gleam&quot;))\n string == &quot;Gleam&quot;\n \n\n ### Notes\n\n This function is an alias for `return`.\n\nSynonyms:\nqcheck.constant\nqcheck constant","url":"qcheck.html#constant"},{"doc":"qcheck","title":"default_config","content":"pub fn default_config() -&gt; Config\n Create a default configuration for property-based testing.\n\n ### Returns\n\n - A `Config` with default settings for test count, max retries, and seed\n\n ### Example\n\n \n let config = default_config()\n \n\nSynonyms:\nqcheck.default_config\nqcheck default_config","url":"qcheck.html#default_config"},{"doc":"qcheck","title":"fixed_length_list_from","content":"pub fn fixed_length_list_from(\n  element_generator: Generator(a),\n  length: Int,\n) -&gt; Generator(List(a))\n Generate fixed-length lists with elements from the given generator.\n\n ### Arguments\n\n - `element_generator`: Generates list elements\n - `length`: The length of the generated lists\n\n ### Returns\n\n A generator that produces fixed-length lists with elements from the given\n generator.\n\n ### Shrinking\n\n Shrinks first on list length, then on list elements, ensuring shrunk lists\n remain within length generator&#39;s range.\n\n ### Example\n\n \n fixed_length_list_from(string(), 5)\n \n\nSynonyms:\nqcheck.fixed_length_list_from\nqcheck fixed_length_list_from","url":"qcheck.html#fixed_length_list_from"},{"doc":"qcheck","title":"fixed_length_string_from","content":"pub fn fixed_length_string_from(\n  generator: Generator(UtfCodepoint),\n  length: Int,\n) -&gt; Generator(String)\n Generate a fixed-length string from the given codepoint generator.\n\n ### Arguments\n\n - `generator`: A generator for codepoints\n - `length`: Number of graphemes in the generated string\n\n ### Returns\n\n A generator that produces strings with the specified number of graphemes\n\n ### Example\n\n \n fixed_length_string_from(codepoint(), 5)\n \n\nSynonyms:\nqcheck.fixed_length_string_from\nqcheck fixed_length_string_from","url":"qcheck.html#fixed_length_string_from"},{"doc":"qcheck","title":"fixed_size_bit_array","content":"pub fn fixed_size_bit_array(size: Int) -&gt; Generator(BitArray)\n Generate fixed-size bit arrays.\n\n ### Warning\n\n This function will generate bit arrays that cause runtime crashes when\n targeting JavaScript.\n\nSynonyms:\nqcheck.fixed_size_bit_array\nqcheck fixed_size_bit_array","url":"qcheck.html#fixed_size_bit_array"},{"doc":"qcheck","title":"fixed_size_bit_array_from","content":"pub fn fixed_size_bit_array_from(\n  value_generator: Generator(Int),\n  bit_size: Int,\n) -&gt; Generator(BitArray)\n Generate fixed-size bit arrays where elements are generated\n using the provided integer generator.\n\n ### Arguments\n\n - `value_generator`: Generators bit array values\n - `bit_size`: Number of bits in the generated bit array\n\n ### Returns\n\n A generator of fixed-size bit arrays\n\n ### Notes\n\n Shrinks on values, not on size\n\n ### Example\n\n \n fixed_size_bit_array_from(bounded_int(0, 255), 64)\n \n\n ### Warning\n\n This function will generate bit arrays that cause runtime crashes when\n targeting JavaScript unless the bit size is a multiple of 8.\n\nSynonyms:\nqcheck.fixed_size_bit_array_from\nqcheck fixed_size_bit_array_from","url":"qcheck.html#fixed_size_bit_array_from"},{"doc":"qcheck","title":"fixed_size_byte_aligned_bit_array","content":"pub fn fixed_size_byte_aligned_bit_array(\n  num_bytes: Int,\n) -&gt; Generator(BitArray)\n Generate byte-aligned bit arrays of the given number of bytes\n\n ### Arguments\n\n - `num_bytes`: Number of bytes for the generated bit array\n\n ### Returns\n\n A generator that produces bit arrays with the specified number of bytes\n\n ### Example\n\n Generate 4-byte bit arrays:\n\n \n fixed_size_byte_aligned_bit_array(4)\n \n\nSynonyms:\nqcheck.fixed_size_byte_aligned_bit_array\nqcheck fixed_size_byte_aligned_bit_array","url":"qcheck.html#fixed_size_byte_aligned_bit_array"},{"doc":"qcheck","title":"fixed_size_byte_aligned_bit_array_from","content":"pub fn fixed_size_byte_aligned_bit_array_from(\n  value_generator: Generator(Int),\n  byte_size: Int,\n) -&gt; Generator(BitArray)\n Generate byte-aligned bit arrays of the given number of bytes from the\n given value generator\n\n ### Arguments\n\n - `value_generator`: Generates the values of the bit array\n - `num_bytes`: Number of bytes for the generated bit array\n\n ### Returns\n\n A generator that produces bit arrays with the specified number of bytes\n according to the given value generator\n\n ### Example\n\n Generate 4-byte bit arrays:\n\n \n fixed_size_byte_aligned_bit_array(bounded_int(0, 255), 16)\n \n\nSynonyms:\nqcheck.fixed_size_byte_aligned_bit_array_from\nqcheck fixed_size_byte_aligned_bit_array_from","url":"qcheck.html#fixed_size_byte_aligned_bit_array_from"},{"doc":"qcheck","title":"fixed_size_utf8_bit_array","content":"pub fn fixed_size_utf8_bit_array(\n  num_codepoints: Int,\n) -&gt; Generator(BitArray)\n Generate a fixed-sized bit array of valid UTF-8 encoded bytes with the\n given number of codepoints.\n\n ### Arguments\n\n - `num_codepoints`: The number of Unicode codepoints represented by the\n generated bit arrays\n\n ### Returns\n\n A generator that produces of fixed-sized bit arrays of UTF-8 encoded bytes\n\n ### Details\n\n - The size is determined by the number of Unicode codepoints, not bytes or\n   bits.\n - If a negative number is provided, it is converted to zero.\n\nSynonyms:\nqcheck.fixed_size_utf8_bit_array\nqcheck fixed_size_utf8_bit_array","url":"qcheck.html#fixed_size_utf8_bit_array"},{"doc":"qcheck","title":"fixed_size_utf8_bit_array_from","content":"pub fn fixed_size_utf8_bit_array_from(\n  codepoint_generator: Generator(UtfCodepoint),\n  num_codepoints: Int,\n) -&gt; Generator(BitArray)\n Generate a fixed-sized bit array of valid UTF-8 encoded bytes with the\n given number of codepoints and values generated from the given codepoint\n generator.\n\n ### Arguments\n\n - `codepoint_generator`: Generates the values\n - `num_codepoints`: The number of Unicode codepoints represented by the\n generated bit arrays\n\n ### Returns\n\n A generator that produces of fixed-sized bit arrays of UTF-8 encoded bytes\n\n ### Details\n\n - The size is determined by the number of Unicode codepoints, not bytes or\n   bits.\n - If a negative number is provided, it is converted to zero.\n\nSynonyms:\nqcheck.fixed_size_utf8_bit_array_from\nqcheck fixed_size_utf8_bit_array_from","url":"qcheck.html#fixed_size_utf8_bit_array_from"},{"doc":"qcheck","title":"float","content":"pub fn float() -&gt; Generator(Float)\n Generate floats with a bias towards smaller values.\n\n Shrinks towards `0.0`.\n\n ### Returns\n\n A generator that produces floating-point numbers\n\nSynonyms:\nqcheck.float\nqcheck float","url":"qcheck.html#float"},{"doc":"qcheck","title":"from_generators","content":"pub fn from_generators(\n  generator: Generator(a),\n  generators: List(Generator(a)),\n) -&gt; Generator(a)\n Choose a generator from a list of generators, then generate a value from\n the selected generator.\n\n ### Arguments\n\n - `generator`: Initial generator to include in the choice\n - `generators`: Additional generators to choose from\n\n ### Returns\n\n A generator that selects and uses one of the provided generators\n\n ### Notes\n\n - Will always produce values since at least one generator is required\n\n ### Example\n\n \n fn mostly_ascii_characters_generator() {\n   from_generators(uppercase_ascii_codepoint(), [\n     lowercase_ascii_codepoint(),\n     uniform_codepoint(),\n   ])\n }\n \n\nSynonyms:\nqcheck.from_generators\nqcheck from_generators","url":"qcheck.html#from_generators"},{"doc":"qcheck","title":"from_weighted_generators","content":"pub fn from_weighted_generators(\n  generator: #(Int, Generator(a)),\n  generators: List(#(Int, Generator(a))),\n) -&gt; Generator(a)\n Choose a generator from a list of weighted generators, then generate a\n value from the selected generator.\n\n ### Arguments\n\n - `generator`: Initial weighted generator (weight and generator)\n - `generators`: Additional weighted generators\n\n ### Returns\n\n A generator that selects and generates values based on the provided\n weights\n\n ### Example\n\n \n from_weighted_generators(#(26, uppercase_ascii_codepoint()), [\n   #(26, lowercase_ascii_codepoint()),\n   #(10, ascii_digit_codepoint()),\n ])\n \n\nSynonyms:\nqcheck.from_weighted_generators\nqcheck from_weighted_generators","url":"qcheck.html#from_weighted_generators"},{"doc":"qcheck","title":"generate","content":"pub fn generate(\n  generator: Generator(a),\n  number_to_generate: Int,\n  seed: Seed,\n) -&gt; #(List(a), Seed)\n Generate a fixed number of random values from a generator.\n\n ### Arguments\n\n - `generator`: The generator to use for creating values\n - `number_to_generate`: Number of values to generate\n - `seed`: Random seed for value generation\n\n ### Returns\n\n A list of generated values, without their associated shrinks\n\n ### Notes\n\n Primarily useful for debugging generator behavior\n\nSynonyms:\nqcheck.generate\nqcheck generate","url":"qcheck.html#generate"},{"doc":"qcheck","title":"generate_tree","content":"pub fn generate_tree(\n  generator: Generator(a),\n  seed: Seed,\n) -&gt; #(Tree(a), Seed)\n Generate a single value and its shrink tree from a generator.\n\n ### Arguments\n\n - `generator`: The generator to use for creating the value\n - `seed`: Random seed for value generation\n\n ### Returns\n\n A tuple containing the generated value&#39;s shrink tree and the next seed\n\n ### Notes\n\n Primarily useful for debugging generator behavior\n\nSynonyms:\nqcheck.generate_tree\nqcheck generate_tree","url":"qcheck.html#generate_tree"},{"doc":"qcheck","title":"generator","content":"pub fn generator(\n  random_generator: Generator(a),\n  tree: fn(a) -&gt; Tree(a),\n) -&gt; qcheck.Generator(a)\n Create a new generator from a random generator and a shrink tree function.\n\n ### Arguments\n\n - `random_generator`: Produces random values of type `a`\n - `tree`: Function that creates a shrink tree for generated values\n\n ### Returns\n\n A new `Generator(a)` that combines random generation and shrinking\n\n ### Notes\n\n This is a low-level function for building custom generators. Prefer using\n built-in generators or combinators like `map`, `bind`, etc.\n\nSynonyms:\nqcheck.generator\nqcheck generator","url":"qcheck.html#generator"},{"doc":"qcheck","title":"generic_bit_array","content":"pub fn generic_bit_array(\n  values_from value_generator: Generator(Int),\n  bit_size_from bit_size_generator: Generator(Int),\n) -&gt; Generator(BitArray)\n Generate bit arrays with configurable values and bit sizes.\n\n ### Arguments\n\n - `values_from`: Generator for bit array contents\n - `bit_size_from`: Generator for bit array size\n\n ### Returns\n\n A bit array generator\n\n ### Example\n\n \n let generator = generic_bit_array(\n   value_generator: bounded_int(0, 255),\n   bit_size_generator: bounded_int(32, 64)\n )\n \n\n ### Warning\n\n This function will generate bit arrays that cause runtime crashes when\n targeting JavaScript.\n\nSynonyms:\nqcheck.generic_bit_array\nqcheck generic_bit_array","url":"qcheck.html#generic_bit_array"},{"doc":"qcheck","title":"generic_byte_aligned_bit_array","content":"pub fn generic_byte_aligned_bit_array(\n  values_from value_generator: Generator(Int),\n  byte_size_from byte_size_generator: Generator(Int),\n) -&gt; Generator(BitArray)\n Generate byte-aligned bit arrays according to the given value generator\n and byte size generator.\n\n ### Arguments\n\n - `value_generator`: Generates the values of the bit array\n - `byte_size_generator`: Generates the number of bytes of the bit array\n\n ### Returns\n\n A byte-aligned bit array generator\n\nSynonyms:\nqcheck.generic_byte_aligned_bit_array\nqcheck generic_byte_aligned_bit_array","url":"qcheck.html#generic_byte_aligned_bit_array"},{"doc":"qcheck","title":"generic_dict","content":"pub fn generic_dict(\n  keys_from key_generator: Generator(a),\n  values_from value_generator: Generator(b),\n  size_from size_generator: Generator(Int),\n) -&gt; Generator(Dict(a, b))\n Generates dictionaries with keys from a key generator, values from a value\n generator, and sizes from a size generator.\n\n ### Arguments\n\n - `keys_from`: Generator for dictionary keys\n - `values_from`: Generator for dictionary values\n - `size_from`: Generator for dictionary size\n\n ### Returns\n\n A generator that produces dictionaries\n\n ### Notes\n\n - The actual size may be less than the generated size due to potential key\n   duplicates\n - Shrinks on size first, then on individual elements\n\n ### Example\n\n \n generic_dict(\n   key_generator: uniform_int(),\n   value_generator: string(),\n   size_generator: small_strictly_positive_int()\n )\n \n\nSynonyms:\nqcheck.generic_dict\nqcheck generic_dict","url":"qcheck.html#generic_dict"},{"doc":"qcheck","title":"generic_list","content":"pub fn generic_list(\n  elements_from element_generator: Generator(a),\n  length_from length_generator: Generator(Int),\n) -&gt; Generator(List(a))\n Generate lists with elements from one generator and lengths from another.\n\n ### Arguments\n\n - `elements_from`: Generates list elements\n - `length_from`: Generates list lengths\n\n ### Returns\n\n A generator that produces lists with:\n - Elements from `elements_from`\n - Lengths from `length_from`\n\n ### Shrinking\n\n Shrinks first on list length, then on list elements, ensuring shrunk lists\n remain within length generator&#39;s range.\n\n ### Example\n\n \n generic_list(string(), small_non_negative_int())\n \n\nSynonyms:\nqcheck.generic_list\nqcheck generic_list","url":"qcheck.html#generic_list"},{"doc":"qcheck","title":"generic_set","content":"pub fn generic_set(\n  elements_from element_generator: Generator(a),\n  size_from size_generator: Generator(Int),\n) -&gt; Generator(Set(a))\n Generates sets with values from an element generator, and sizes from a size\n generator.\n\n ### Arguments\n\n - `elements_from`: Generator for set elements\n - `size_from`: Generator for set size\n\n ### Returns\n\n A generator that produces sets\n\n ### Notes\n\n - The actual size may be less than the generated size due to potential\n   duplicates\n - Shrinks on size first, then on individual elements\n\n ### Example\n\n \n generic_set(\n   value_generator: string(),\n   size_generator: small_strictly_positive_int()\n )\n \n\nSynonyms:\nqcheck.generic_set\nqcheck generic_set","url":"qcheck.html#generic_set"},{"doc":"qcheck","title":"generic_string","content":"pub fn generic_string(\n  codepoints_from codepoint_generator: Generator(UtfCodepoint),\n  length_from length_generator: Generator(Int),\n) -&gt; Generator(String)\n Generate a string from the given codepoint generator and the given length\n generator.\n\n ### Arguments\n\n - `codepoint_generator`: A generator for codepoints\n - `length_generator`: A generator to determine number of graphemes in the\n      generated strings\n\n ### Returns\n\n A string generator\n\n ### Example\n\n \n generic_string(ascii_digit_codepoint(), bounded_int(8, 15))\n \n\nSynonyms:\nqcheck.generic_string\nqcheck generic_string","url":"qcheck.html#generic_string"},{"doc":"qcheck","title":"generic_utf8_bit_array","content":"pub fn generic_utf8_bit_array(\n  codepoints_from codepoint_generator: Generator(UtfCodepoint),\n  codepoint_size_from num_codepoints_generator: Generator(Int),\n) -&gt; Generator(Generator(List(UtfCodepoint)))\n Generate bit arrays of UTF-8 encoded bytes with configurable values\n and number of codepoints.\n\n ### Arguments\n\n - `codepoints_from`: Generates the codepoint values of the resulting\n     bit arrays\n - `codepoint_size_from`: Generates sizes in number of codepoints represented\n     by the resulting bit arrays\n\n ### Returns\n\n A generator of bit arrays of valid UTF-8 encoded bytes\n\nSynonyms:\nqcheck.generic_utf8_bit_array\nqcheck generic_utf8_bit_array","url":"qcheck.html#generic_utf8_bit_array"},{"doc":"qcheck","title":"given","content":"pub fn given(\n  generator: Generator(a),\n  property: fn(a) -&gt; Nil,\n) -&gt; Nil\n Test a property against generated test cases using the default\n configuration.\n\n ### Arguments\n\n - `generator`: Creates test inputs\n - `property`: The property to verify\n\n ### Returns\n\n - `Nil` if all test cases pass (the property returns `Nil`)\n - Panics if any test case fails (the property panics)\n\nSynonyms:\nqcheck.given\nqcheck given","url":"qcheck.html#given"},{"doc":"qcheck","title":"list_from","content":"pub fn list_from(\n  element_generator: Generator(a),\n) -&gt; Generator(List(a))\n Generate lists with elements from the given generator and the default\n length generator.\n\n ### Arguments\n\n - `element_generator`: Generates list elements\n\n ### Returns\n\n A generator that produces lists with elements from the given generator\n\n ### Shrinking\n\n Shrinks first on list length, then on list elements.\n\n ### Example\n\n \n list_from(string())\n \n\nSynonyms:\nqcheck.list_from\nqcheck list_from","url":"qcheck.html#list_from"},{"doc":"qcheck","title":"lowercase_ascii_codepoint","content":"pub fn lowercase_ascii_codepoint() -&gt; Generator(UtfCodepoint)\n Generate lowercase ASCII letters.\n\n ### Returns\n\n A generator that produces lowercase letters from `a` to `z` as codepoints\n\n ### Example\n\n \n string_from(lowercase_ascii_codepoint())\n \n\nSynonyms:\nqcheck.lowercase_ascii_codepoint\nqcheck lowercase_ascii_codepoint","url":"qcheck.html#lowercase_ascii_codepoint"},{"doc":"qcheck","title":"map","content":"pub fn map(\n  generator: Generator(a),\n  f: fn(a) -&gt; b,\n) -&gt; Generator(b)\n Transform a generator by applying a function to each generated value.\n\n ### Arguments\n\n - `generator`: The original generator to transform\n - `f`: Function to apply to each generated value\n\n ### Returns\n\n A new generator that produces values transformed by `f`, with shrinking\n behavior derived from the original generator\n\n ### Examples\n\n \n let even_number_generator = map(uniform_int(), fn(n) { 2 * n })\n \n\n With `use`:\n\n \n let even_number_generator = {\n   use n &lt;- map(uniform_int())\n   2 * n\n }\n \n\nSynonyms:\nqcheck.map\nqcheck map","url":"qcheck.html#map"},{"doc":"qcheck","title":"map2","content":"pub fn map2(\n  g1: Generator(a),\n  g2: Generator(b),\n  f: fn(a, b) -&gt; c,\n) -&gt; Generator(c)\n Transform two generators by applying a function to their generated values.\n\n ### Arguments\n\n - `g1`: First generator to provide input\n - `g2`: Second generator to provide input\n - `f`: Function to apply to generated values from `g1` and `g2`\n\n ### Returns\n\n A new generator that produces values by applying `f` to values from `g1`\n and `g2`\n\n ### Example\n\n \n use year, month &lt;- map2(bounded_int(0, 9999), bounded_int(1, 12))\n int.to_string(year) &lt;&gt; &quot;-&quot; &lt;&gt; int.to_string(month)\n \n\nSynonyms:\nqcheck.map2\nqcheck map2","url":"qcheck.html#map2"},{"doc":"qcheck","title":"map3","content":"pub fn map3(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n  f: fn(a, b, c) -&gt; d,\n) -&gt; Generator(d)\n Transform three generators by applying a function to their generated values.\n\n See docs for [`map2`](#map2).\n\nSynonyms:\nqcheck.map3\nqcheck map3","url":"qcheck.html#map3"},{"doc":"qcheck","title":"map4","content":"pub fn map4(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n  g4: Generator(d),\n  f: fn(a, b, c, d) -&gt; e,\n) -&gt; Generator(e)\n Transform four generators by applying a function to their generated values.\n\n See docs for [`map2`](#map2).\n\nSynonyms:\nqcheck.map4\nqcheck map4","url":"qcheck.html#map4"},{"doc":"qcheck","title":"map5","content":"pub fn map5(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n  g4: Generator(d),\n  g5: Generator(e),\n  f: fn(a, b, c, d, e) -&gt; f,\n) -&gt; Generator(f)\n Transform five generators by applying a function to their generated values.\n\n See docs for [`map2`](#map2).\n\nSynonyms:\nqcheck.map5\nqcheck map5","url":"qcheck.html#map5"},{"doc":"qcheck","title":"map6","content":"pub fn map6(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n  g4: Generator(d),\n  g5: Generator(e),\n  g6: Generator(f),\n  f: fn(a, b, c, d, e, f) -&gt; g,\n) -&gt; Generator(g)\n Transform six generators by applying a function to their generated values.\n\n See docs for [`map2`](#map2).\n\nSynonyms:\nqcheck.map6\nqcheck map6","url":"qcheck.html#map6"},{"doc":"qcheck","title":"nil","content":"pub fn nil() -&gt; Generator(Nil)\n Generate a constant `Nil` value.\n\n ### Returns\n\n A `Generator` that always returns `Nil` and does not shrink\n\nSynonyms:\nqcheck.nil\nqcheck nil","url":"qcheck.html#nil"},{"doc":"qcheck","title":"non_empty_bit_array","content":"pub fn non_empty_bit_array() -&gt; Generator(BitArray)\n Generate non-empty bit arrays.\n\n ### Returns\n\n A generator of non-empty bit arrays\n\n ### Warning\n\n This function will generate bit arrays that cause runtime crashes when\n targeting JavaScript.\n\nSynonyms:\nqcheck.non_empty_bit_array\nqcheck non_empty_bit_array","url":"qcheck.html#non_empty_bit_array"},{"doc":"qcheck","title":"non_empty_byte_aligned_bit_array","content":"pub fn non_empty_byte_aligned_bit_array() -&gt; Generator(BitArray)\n Generate non-empty byte-aligned bit arrays.\n\nSynonyms:\nqcheck.non_empty_byte_aligned_bit_array\nqcheck non_empty_byte_aligned_bit_array","url":"qcheck.html#non_empty_byte_aligned_bit_array"},{"doc":"qcheck","title":"non_empty_string","content":"pub fn non_empty_string() -&gt; Generator(String)\n Generate non-empty strings with the default codepoint and length generators.\n\n ### Example\n\n \n use string &lt;- given(string())\n string.length(string) &gt; 0\n \n\nSynonyms:\nqcheck.non_empty_string\nqcheck non_empty_string","url":"qcheck.html#non_empty_string"},{"doc":"qcheck","title":"non_empty_string_from","content":"pub fn non_empty_string_from(\n  codepoint_generator: Generator(UtfCodepoint),\n) -&gt; Generator(String)\n Generate non-empty strings with the given codepoint generator and default\n length generator.\n\n ### Example\n\n \n non_empty_string_from(alphanumeric_ascii_codepoint())\n \n\nSynonyms:\nqcheck.non_empty_string_from\nqcheck non_empty_string_from","url":"qcheck.html#non_empty_string_from"},{"doc":"qcheck","title":"non_empty_utf8_bit_array","content":"pub fn non_empty_utf8_bit_array() -&gt; Generator(BitArray)\n Generate non-empty bit arrays of valid UTF-8 bytes.\n\nSynonyms:\nqcheck.non_empty_utf8_bit_array\nqcheck non_empty_utf8_bit_array","url":"qcheck.html#non_empty_utf8_bit_array"},{"doc":"qcheck","title":"option_from","content":"pub fn option_from(\n  generator: Generator(a),\n) -&gt; Generator(Option(a))\n Create a generator for `Option` values.\n\n ### Arguments\n\n - `generator`: Generator for the inner value type\n\n ### Returns\n\n A generator that produces `Option` values, shrinking towards `None` first,\n then towards the shrinks of the input generator\n\n ### Example\n\n \n option_from(string())\n \n\nSynonyms:\nqcheck.option_from\nqcheck option_from","url":"qcheck.html#option_from"},{"doc":"qcheck","title":"parameter","content":"pub fn parameter(f: fn(a) -&gt; b) -&gt; fn(a) -&gt; b\n Support for constructing curried functions for the applicative style of\n generator composition.\n\n ### Example\n\n \n type Box {\n   Box(x: Int, y: Int, w: Int, h: Int)\n }\n\n fn box_generator() {\n   return({\n     use x &lt;- parameter\n     use y &lt;- parameter\n     use w &lt;- parameter\n     use h &lt;- parameter\n     Box(x:, y:, w:, h:)\n   })\n   |&gt; apply(bounded_int(-100, 100))\n   |&gt; apply(bounded_int(-100, 100))\n   |&gt; apply(bounded_int(1, 100))\n   |&gt; apply(bounded_int(1, 100))\n }\n \n\nSynonyms:\nqcheck.parameter\nqcheck parameter","url":"qcheck.html#parameter"},{"doc":"qcheck","title":"printable_ascii_codepoint","content":"pub fn printable_ascii_codepoint() -&gt; Generator(UtfCodepoint)\n Generate printable ASCII characters with a bias towards alphanumeric\n characters.\n\n ### Returns\n\n A generator that produces printable ASCII characters as codepoints\n\n ### Example\n\n \n string_from(printable_ascii_codepoint())\n \n\nSynonyms:\nqcheck.printable_ascii_codepoint\nqcheck printable_ascii_codepoint","url":"qcheck.html#printable_ascii_codepoint"},{"doc":"qcheck","title":"random_seed","content":"pub fn random_seed() -&gt; Seed\n Create a new randomly-generated seed.\n\n ### Returns\n\n A `Seed` value that can be used to configure non-deterministic test generation\n\n ### Example\n\n \n let config = config(test_count: 10_000, max_retries: 1, seed: random_seed())\n \n\nSynonyms:\nqcheck.random_seed\nqcheck random_seed","url":"qcheck.html#random_seed"},{"doc":"qcheck","title":"return","content":"pub fn return(a: a) -&gt; Generator(a)\n Create a generator that always returns the same value and does not shrink.\n\n ### Arguments\n\n - `a`: The value to be consistently generated\n\n ### Returns\n\n A `Generator` that produces the same input for all test cases\n\n ### Example\n\n \n use string &lt;- given(return(&quot;Gleam&quot;))\n string == &quot;Gleam&quot;\n \n\nSynonyms:\nqcheck.return\nqcheck return","url":"qcheck.html#return"},{"doc":"qcheck","title":"run","content":"pub fn run(\n  config: Config,\n  generator: Generator(a),\n  property: fn(a) -&gt; Nil,\n) -&gt; Nil\n Test a property against generated test cases using the provided\n configuration.\n\n ### Arguments\n\n - `config`: Settings for test execution\n - `generator`: Creates test inputs\n - `property`: The property to verify\n\n ### Returns\n\n - `Nil` if all test cases pass (the property returns `Nil`)\n - Panics if any test case fails (the property panics)\n\nSynonyms:\nqcheck.run\nqcheck run","url":"qcheck.html#run"},{"doc":"qcheck","title":"seed","content":"pub fn seed(n: Int) -&gt; Seed\n Create a new seed from a provided integer.\n\n ### Arguments\n\n - `n`: Integer to create the seed from\n\n ### Returns\n\n A `Seed` value that can be used to configure deterministic test generation\n\n ### Example\n\n \n let config = default_config() |&gt; with_seed(seed(124))\n \n\nSynonyms:\nqcheck.seed\nqcheck seed","url":"qcheck.html#seed"},{"doc":"qcheck","title":"sized_from","content":"pub fn sized_from(\n  sized_generator: fn(Int) -&gt; Generator(a),\n  size_generator: Generator(Int),\n) -&gt; Generator(a)\n Creates a generator by first generating a size using the provided\n `size_generator`, then passing that size to the `sized_generator` to\n produce a value.\n\n Shrinks on the size first, then on the generator.\n\n ### Arguments\n\n - `sized_generator`: A generator function that takes a size and produces a\n     value\n - `size_generator`: A generator for creating the size input\n\n ### Returns\n\n A generator that first produces a size, then uses that size to generate a\n value\n\n ### Example\n\n Create a bit arrays whose bit size is from 10 to 20.\n\n \n fixed_size_bit_array() |&gt; sized_from(bounded_int(10, 20))\n \n\nSynonyms:\nqcheck.sized_from\nqcheck sized_from","url":"qcheck.html#sized_from"},{"doc":"qcheck","title":"small_non_negative_int","content":"pub fn small_non_negative_int() -&gt; Generator(Int)\n Generate small non-negative integers, well-suited for modeling sized\n elements like lists or strings.\n\n Shrinks towards `0`.\n\n ### Returns\n\n A generator for small, non-negative integers\n\n ### Example\n\n \n generic_string(bounded_codepoint(0, 255), small_non_negative_int())\n \n\nSynonyms:\nqcheck.small_non_negative_int\nqcheck small_non_negative_int","url":"qcheck.html#small_non_negative_int"},{"doc":"qcheck","title":"small_strictly_positive_int","content":"pub fn small_strictly_positive_int() -&gt; Generator(Int)\n Generate small, strictly positive integers, well-suited for modeling sized\n elements like lists or strings.\n\n Shrinks towards `0`.\n\n ### Returns\n\n A generator for small, strictly positive integers\n\n ### Example\n\n \n generic_string(bounded_codepoint(0, 255), small_strictly_positive_int())\n \n\nSynonyms:\nqcheck.small_strictly_positive_int\nqcheck small_strictly_positive_int","url":"qcheck.html#small_strictly_positive_int"},{"doc":"qcheck","title":"string","content":"pub fn string() -&gt; Generator(String)\n Generate strings with the default codepoint and length generators.\n\n ### Example\n\n \n use string &lt;- given(string())\n string.length(string) == string.length(string &lt;&gt; &quot;!&quot;) + 1\n \n\nSynonyms:\nqcheck.string\nqcheck string","url":"qcheck.html#string"},{"doc":"qcheck","title":"string_from","content":"pub fn string_from(\n  codepoint_generator: Generator(UtfCodepoint),\n) -&gt; Generator(String)\n Generate strings with the given codepoint generator and default length\n generator.\n\n ### Example\n\n \n string_from(ascii_digit_codepoint())\n \n\nSynonyms:\nqcheck.string_from\nqcheck string_from","url":"qcheck.html#string_from"},{"doc":"qcheck","title":"then","content":"pub fn then(\n  generator: Generator(a),\n  f: fn(a) -&gt; Generator(b),\n) -&gt; Generator(b)\n Transform a generator by applying a function that returns another\n generator to each generated value.\n\n Unlike `map`, this allows for a dependency on the resulting generator and\n the original generated values.\n\n (`then` is an alias for `bind`.)\n\n ### Arguments\n\n - `generator`: A generator that creates a value of type `a`\n - `f`: A function that takes a value of type `a` and returns a generator\n     of type `b`\n\n ### Returns\n\n A generator that first generates a value of type `a`, then uses that value\n to generate a value of type `b`\n\n ### Examples\n\n Say you wanted to generate a valid date in string form, like `&quot;2025-01-30&quot;`.\n In order to generate a valid day, you need both the month (some months have\n 31 days, other have fewer) and also the year (since the year affects the max\n days in February). So, before you can generate a valid day, you must first\n generate a year and a month. You could imagine a set of functions like this:\n\n \n fn date_generator() -&gt; Generator(String) {\n   use #(year, month) &lt;- then(tuple2(year_generator(), month_generator()))\n   use day &lt;- map(day_generator(year:, month:))\n\n   int.to_string(year)\n   &lt;&gt; &quot;-&quot;\n   &lt;&gt; int.to_string(month)\n   &lt;&gt; &quot;-&quot;\n   &lt;&gt; int.to_string(day)\n }\n\n // Note how the day generator depends on the value of `year` and `month`.\n fn day_generator(year year, month month) -&gt; Generator(Int) {\n   todo\n }\n\n fn year_generator() -&gt; Generator(Int) {\n   todo\n }\n\n fn month_generator() -&gt; Generator(Int) {\n   todo\n }\n \n\n Another situation in which you would need `then` is if you needed to\n generate departure and arrival times. We will say a pair of departure and\n arrival times is valid if the departure time is before the arrival time.\n That means we cannot generate an arrival time without first generating a\n departure time. Here is how that might look:\n\n \n fn departure_and_arrival_generator() {\n   use departure_time &lt;- then(departure_time_generator())\n   use arrival_time &lt;- map(arrival_time_generator(departure_time))\n   #(departure_time, arrival_time)\n }\n\n fn departure_time_generator() {\n   todo\n }\n\n fn arrival_time_generator(departure_time) {\n   todo\n }\n \n\nSynonyms:\nqcheck.then\nqcheck then","url":"qcheck.html#then"},{"doc":"qcheck","title":"tuple2","content":"pub fn tuple2(\n  g1: Generator(a),\n  g2: Generator(b),\n) -&gt; Generator(#(a, b))\n Generate a tuple of two values using the provided generators.\n\n ### Arguments\n\n - `g1`: Generator for the first tuple element\n - `g2`: Generator for the second tuple element\n\n ### Returns\n\n A generator that produces a tuple of two values, one from each input\n generator\n\n ### Example\n\n \n let point_generator = tuple2(float(), float())\n \n\nSynonyms:\nqcheck.tuple2\nqcheck tuple2","url":"qcheck.html#tuple2"},{"doc":"qcheck","title":"tuple3","content":"pub fn tuple3(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n) -&gt; Generator(#(a, b, c))\n Generate a tuple of three values using the provided generators.\n\n See docs for [`tuple2`](#tuple2).\n\nSynonyms:\nqcheck.tuple3\nqcheck tuple3","url":"qcheck.html#tuple3"},{"doc":"qcheck","title":"tuple4","content":"pub fn tuple4(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n  g4: Generator(d),\n) -&gt; Generator(#(a, b, c, d))\n Generate a tuple of four values using the provided generators.\n\n See docs for [`tuple2`](#tuple2).\n\nSynonyms:\nqcheck.tuple4\nqcheck tuple4","url":"qcheck.html#tuple4"},{"doc":"qcheck","title":"tuple5","content":"pub fn tuple5(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n  g4: Generator(d),\n  g5: Generator(e),\n) -&gt; Generator(#(a, b, c, d, e))\n Generate a tuple of five values using the provided generators.\n\n See docs for [`tuple2`](#tuple2).\n\nSynonyms:\nqcheck.tuple5\nqcheck tuple5","url":"qcheck.html#tuple5"},{"doc":"qcheck","title":"tuple6","content":"pub fn tuple6(\n  g1: Generator(a),\n  g2: Generator(b),\n  g3: Generator(c),\n  g4: Generator(d),\n  g5: Generator(e),\n  g6: Generator(f),\n) -&gt; Generator(#(a, b, c, d, e, f))\n Generate a tuple of six values using the provided generators.\n\n See docs for [`tuple2`](#tuple2).\n\nSynonyms:\nqcheck.tuple6\nqcheck tuple6","url":"qcheck.html#tuple6"},{"doc":"qcheck","title":"uniform_codepoint","content":"pub fn uniform_codepoint() -&gt; Generator(UtfCodepoint)\n Generate Unicode codepoints.\n\n ### Returns\n\n A generator that creates Unicode codepoints across the valid range\n\n ### Notes\n\n - Generates codepoints from U+0000 to U+10FFFF\n - Uses ASCII lowercase &#39;a&#39; as a shrink target\n\n ### Example\n\n \n string_from(uniform_codepoint())\n \n\nSynonyms:\nqcheck.uniform_codepoint\nqcheck uniform_codepoint","url":"qcheck.html#uniform_codepoint"},{"doc":"qcheck","title":"uniform_int","content":"pub fn uniform_int() -&gt; Generator(Int)\n Generate uniformly distributed integers across a large range.\n\n ### Details\n\n - Shrinks generated values towards `0`\n - Not likely to hit interesting or corner cases\n\n ### Returns\n\n A generator of integers with uniform distribution\n\n ### Example\n\n \n let positive_int_generator = {\n   use n &lt;- map(uniform_int())\n   int.absolute_value(n)\n }\n \n\nSynonyms:\nqcheck.uniform_int\nqcheck uniform_int","url":"qcheck.html#uniform_int"},{"doc":"qcheck","title":"uniform_printable_ascii_codepoint","content":"pub fn uniform_printable_ascii_codepoint() -&gt; Generator(\n  UtfCodepoint,\n)\n Uniformly generate printable ASCII characters.\n\n ### Returns\n\n A generator that produces printable ASCII characters as codepoints\n\n ### Example\n\n \n string_from(uniform_printable_ascii_codepoint())\n \n\nSynonyms:\nqcheck.uniform_printable_ascii_codepoint\nqcheck uniform_printable_ascii_codepoint","url":"qcheck.html#uniform_printable_ascii_codepoint"},{"doc":"qcheck","title":"uppercase_ascii_codepoint","content":"pub fn uppercase_ascii_codepoint() -&gt; Generator(UtfCodepoint)\n Generate uppercase ASCII letters.\n\n ### Returns\n\n A generator that produces uppercase letters from `A` to `Z` as codepoints\n\n ### Example\n\n \n string_from(uppercase_ascii_codepoint())\n \n\nSynonyms:\nqcheck.uppercase_ascii_codepoint\nqcheck uppercase_ascii_codepoint","url":"qcheck.html#uppercase_ascii_codepoint"},{"doc":"qcheck","title":"utf8_bit_array","content":"pub fn utf8_bit_array() -&gt; Generator(BitArray)\n Generate bit arrays of valid UTF-8 bytes.\n\nSynonyms:\nqcheck.utf8_bit_array\nqcheck utf8_bit_array","url":"qcheck.html#utf8_bit_array"},{"doc":"qcheck","title":"with_max_retries","content":"pub fn with_max_retries(\n  config: Config,\n  max_retries: Int,\n) -&gt; Config\n Set the maximum number of retries for a property test.\n\n ### Arguments\n\n - `config`: The current configuration\n - `max_retries`: Maximum number of retries allowed.  If `max_retries &lt; 0`,\n     uses the default max retries.\n\n ### Returns\n\n A new `Config` with the specified maximum retries\n\n ### Example\n\n \n let config = default_config() |&gt; with_max_retries(100)\n \n\nSynonyms:\nqcheck.with_max_retries\nqcheck with_max_retries","url":"qcheck.html#with_max_retries"},{"doc":"qcheck","title":"with_seed","content":"pub fn with_seed(config: Config, seed: Seed) -&gt; Config\n Set the random seed for reproducible test case generation.\n\n ### Arguments\n\n - `config`: The current configuration\n - `seed`: Seed value for random number generation\n\n ### Returns\n\n A new `Config` with the specified random seed\n\n ### Example\n\n \n let config = default_config() |&gt; with_seed(seed(124))\n \n\nSynonyms:\nqcheck.with_seed\nqcheck with_seed","url":"qcheck.html#with_seed"},{"doc":"qcheck","title":"with_test_count","content":"pub fn with_test_count(config: Config, test_count: Int) -&gt; Config\n Set the number of test cases to run in a property test.\n\n ### Arguments\n\n - `config`: The current configuration\n - `test_count`: Number of test cases to generate.  If `test_count &lt;= 0`,\n     uses the default test count.\n\n ### Returns\n\n A new `Config` with the specified test count\n\n ### Example\n\n \n let config = default_config() |&gt; with_test_count(10_000)\n \n\nSynonyms:\nqcheck.with_test_count\nqcheck with_test_count","url":"qcheck.html#with_test_count"},{"doc":"qcheck","title":"qcheck","content":" QuickCheck-inspired property-based testing with integrated shrinking\n\n\n ## Overview\n\n Rather than specifying test cases manually, you describe the invariants\n that values of a given type must satisfy (&quot;properties&quot;). Then, generators\n generate lots of values (test cases) on which the properties are checked.\n Finally, if a value is found for which a given property does not hold, that\n value is &quot;shrunk&quot; in order to find an nice, informative counter-example\n that is presented to you.\n\n This module has functions for running and configuring property tests as\n well as generating random values (with shrinking) to drive those tests.\n\n For full usage examples, see the project README.\n\n ## API\n\n ### Running Tests\n\n - [given](#given)\n - [run](#run)\n\n ### Configuring and Seeding\n\n - The [Config](#Config) type\n - [config](#config)\n - [default_config](#default_config)\n - [with_seed](#with_seed)\n - [with_test_count](#with_test_count)\n - [with_max_retries](#with_max_retries)\n\n - The [Seed](#Seed) type\n - [seed](#seed)\n - [random_seed](#random_seed)\n\n ### Low-Level Construction\n\n - The [Generator](#Generator) type\n - [generator](#generator)\n\n ### Combinators and Other Utilities\n\n - [return](#return) (and [constant](#constant))\n - [bind](#bind) (and [then](#then))\n - [apply](#apply)\n - [parameter](#parameter)\n - [map](#map)\n - [map2](#map2)\n - [map3](#map3)\n - [map4](#map4)\n - [map5](#map5)\n - [map6](#map6)\n - [from_generators](#from_generators)\n - [from_weighted_generators](#from_weighted_generators)\n - [sized_from](#sized_from)\n\n ### Generator Categories\n\n There are a few different &quot;categories&quot; of generator.\n\n - Some types have generators named after the type.\n   - These give a distribution of values that is a reasonable default for test generation.\n   - E.g., `string`, `float`, `bit_array`.\n - Generic generators\n   - These are fully specified, or &quot;generic&quot;, and you must provide generators for values and sizes.\n   - E.g., `generic_string`, `generic_list`.\n - Fixed size/length generators\n   - These take a size or length parameter as appropriate for the type, and generate values of that size (when possible).\n   - These generators use the default value generator.\n   - E.g., `fixed_length_string`, `fixed_length_list`\n - Non-empty generators\n   - These generate collections with length or size of at least one\n   - E.g., `non_empty_string`, `non_empty_bit_array`\n - From other generators\n   - The `_from` suffix means that another generator is used to generate values\n   - E.g., `string_from`, `list_from`\n - Mixing and matching\n   - Some generators mix and match the above categories\n   - E.g., `fixed_length_list_from`, `non_empty_string_from`\n\n ### Numeric Generators\n\n #### Ints\n\n - [uniform_int](#uniform_int)\n - [bounded_int](#bounded_int)\n - [small_non_negative_int](#small_non_negative_int)\n - [small_strictly_positive_int](#small_strictly_positive_int)\n\n #### Floats\n\n - [float](#float)\n - [bounded_float](#bounded_float)\n\n ### Codepoint and String Generators\n\n The main purpose of codepoint generators is to use them to generate\n strings.\n\n #### Codepoints\n\n - [codepoint](#codepoint)\n - [uniform_codepoint](#uniform_codepoint)\n - [bounded_codepoint](#bounded_codepoint)\n - [codepoint_from_ints](#codepoint_from_ints)\n - [codepoint_from_strings](#codepoint_from_strings)\n\n ##### ASCII Codepoints\n\n - [uppercase_ascii_codepoint](#uppercase_ascii_codepoint)\n - [lowercase_ascii_codepoint](#lowercase_ascii_codepoint)\n - [ascii_digit_codepoint](#ascii_digit_codepoint)\n - [alphabetic_ascii_codepoint](#alphabetic_ascii_codepoint)\n - [alphanumeric_ascii_codepoint](#alphanumeric_ascii_codepoint)\n - [printable_ascii_codepoint](#printable_ascii_codepoint)\n - [ascii_whitespace_codepoint](#ascii_whitespace_codepoint)\n - [uniform_printable_ascii_codepoint](#uniform_printable_ascii_codepoint)\n\n #### Strings\n\n String generators are built from codepoint generators.\n\n - [string](#string)\n - [string_from](#string_from)\n - [non_empty_string](#non_empty_string)\n - [non_empty_string_from](#non_empty_string_from)\n - [generic_string](#generic_string)\n - [fixed_length_string_from](#fixed_length_string_from)\n\n ### Bit Array Generators\n\n Bit array values come from integers, and handle sizes and shrinking in a\n reasonable way given that values in the bit array are connected to the\n size of the bit array in certain situations.\n\n These functions will generate bit arrays that cause runtime crashes when\n targeting JavaScript.\n\n - [bit_array](#bit_array)\n - [non_empty_bit_array](#non_empty_bit_array)\n - [fixed_size_bit_array](#fixed_size_bit_array)\n - [fixed_size_bit_array_from](#fixed_size_bit_array_from)\n - [generic_bit_array](#generic_bit_array)\n\n #### Byte-aligned bit arrays\n\n Byte-aligned bit arrays always have a size that is a multiple of 8.\n\n These bit arrays work on the JavaScript target.\n\n - [byte_aligned_bit_array](#byte_aligned_bit_array)\n - [non_empty_byte_aligned_bit_array](#non_empty_byte_aligned_bit_array)\n - [fixed_size_byte_aligned_bit_array](#fixed_size_byte_aligned_bit_array)\n - [fixed_size_byte_aligned_bit_array_from](#fixed_size_byte_aligned_bit_array_from)\n - [generic_byte_aligned_bit_array](#generic_byte_aligned_bit_array)\n\n #### UTF-8 Encoded Bit Arrays\n\n Bit arrays where the values are always valid utf-8 encoded bytes.\n\n These bit arrays work on the JavaScript target.\n\n - [utf8_bit_array](#utf8_bit_array)\n - [non_empty_utf8_bit_array](#non_empty_utf8_bit_array)\n - [fixed_size_utf8_bit_array](#fixed_size_utf8_bit_array)\n - [fixed_size_utf8_bit_array_from](#fixed_size_utf8_bit_array_from)\n - [generic_utf8_bit_array](#generic_utf8_bit_array)\n\n ### Collection Generators\n\n #### Lists\n\n - [list_from](#list_from)\n - [fixed_length_list_from](#fixed_length_list_from)\n - [generic_list](#generic_list)\n\n #### Dictionaries\n\n - [generic_dict](#generic_dict)\n\n #### Sets\n\n - [generic_set](#generic_set)\n\n #### Tuples\n\n - [tuple2](#tuple2)\n - [tuple3](#tuple3)\n - [tuple4](#tuple4)\n - [tuple5](#tuple5)\n - [tuple6](#tuple6)\n\n ### Other Generators\n\n - [bool](#bool)\n - [nil](#nil)\n - [option_from](#option_from)\n\n ### Debug Generators\n\n These functions aren&#39;t meant to be used directly in your tests.  They are\n provided to help debug or investigate what values and shrinks that a\n generator produces.\n\n - [generate](#generate)\n - [generate_tree](#generate_tree)\n\n ## Notes\n\n The exact distributions of individual generators are considered an\n implementation detail, and may change without a major version update.\n For example, if the `option` generator currently produced `None`\n approximately 25% of the time, but that distribution was changed to produce\n `None` 50% of the time instead, that would _not_ be considered a breaking\n change.\n","url":"qcheck.html"}]);